"""parse_plan() tests."""

# import os.path
from os import path as osp

import pytest

from infrahouse_toolkit.terraform import parse_plan
from infrahouse_toolkit.terraform.status import RunResult

# pylint: disable=line-too-long


@pytest.mark.parametrize(
    "output, expected_result",
    [
        (
            "",
            (RunResult(None, None, None), RunResult(None, None, None)),
        ),
        (None, (RunResult(None, None, None), RunResult(None, None, None))),
        (
            open(osp.join(osp.dirname(osp.realpath(__file__)), "plans/plan-0-0-0.stdout")).read(),
            (
                RunResult(0, 0, 0),
                RunResult(
                    [],
                    [],
                    [],
                ),
            ),
        ),
        (
            open(osp.join(osp.dirname(osp.realpath(__file__)), "plans/plan-2-0-0.stdout")).read(),
            (
                RunResult(2, 0, 0),
                RunResult(
                    [
                        'module.repos["test"].github_repository.repo',
                        'module.repos["test"].github_team_repository.dev',
                    ],
                    [],
                    [],
                ),
            ),
        ),
        (
            open(osp.join(osp.dirname(osp.realpath(__file__)), "plans/plan-2-1-2.stdout")).read(),
            (
                RunResult(2, 1, 2),
                RunResult(
                    [
                        'module.repos["test"].github_repository.repo',
                        'module.repos["test"].github_team_repository.dev',
                    ],
                    ['module.repos["infrahouse-toolkit"].github_repository.repo'],
                    [
                        'module.repos["cookiecutter-github-control"].github_repository.repo',
                        'module.repos["cookiecutter-github-control"].github_team_repository.dev',
                    ],
                ),
            ),
        ),
        (
            open(osp.join(osp.dirname(osp.realpath(__file__)), "plans/plan-3-0-0.stdout")).read(),
            (
                RunResult(3, 0, 0),
                RunResult(
                    [
                        'module.repos["terraform-aws-ci-cd"].github_actions_secret.secret["GH_TOKEN"]',
                        'module.repos["terraform-aws-ci-cd"].github_repository.repo',
                        'module.repos["terraform-aws-ci-cd"].github_team_repository.dev',
                    ],
                    [],
                    [],
                ),
            ),
        ),
    ],
)
def test_parse_plan(output, expected_result):
    """
    parse_plan() returns valid result.

    :param output: terraform plan output.
    :param expected_result: expected result.
    """
    result = parse_plan(output)
    assert result == expected_result
    assert isinstance(result[0], RunResult)
    assert isinstance(result[1], RunResult)



"""
.. topic:: ``ih-secrets set``

    A ``ih-secrets set`` subcommand.

    See ``ih-secrets set`` for more details.
"""

import sys
from logging import getLogger

import click
from botocore.exceptions import ClientError

from infrahouse_toolkit.cli.lib import read_from_file_or_prompt

LOG = getLogger(__name__)


@click.command(name="set")
@click.argument("secret")
@click.argument("path", nargs=-1)
@click.pass_context
def cmd_set(ctx, secret, path):
    """
    Set value to a secret.

    Optionally the value may be given via a local file specified by a path argument.

    \b
    ih-secrets set mysecret /path/to/file_with_value

    if the path is omitted, a user will be prompt for the value.
    """
    secretsmanager_client = ctx.obj["secretsmanager_client"]
    aws_config = ctx.obj["aws_config"]
    try:
        value = read_from_file_or_prompt(path[0] if path else None)
        LOG.debug("Secret value: %s", value)
        secretsmanager_client.put_secret_value(SecretId=secret, SecretString=value)
        LOG.info("Value of %s is successfully set.", secret)

    except ClientError as err:
        LOG.exception(err)
        LOG.error("Try to run ih-secrets with --aws-profile option.")
        LOG.error("Available profiles:\n\t%s", "\n\t".join(aws_config.profiles))
        sys.exit(1)



"""
.. topic:: ``ih-elastic snapshots``

    A ``ih-elastic snapshots`` subcommand.

    See ``ih-elastic snapshots --help`` for more details.
"""

import logging

import click

from infrahouse_toolkit.cli.ih_elastic.cmd_snapshots.cmd_create import cmd_create
from infrahouse_toolkit.cli.ih_elastic.cmd_snapshots.cmd_create_repository import (
    cmd_create_repository,
)
from infrahouse_toolkit.cli.ih_elastic.cmd_snapshots.cmd_delete_repository import (
    cmd_delete_repository,
)
from infrahouse_toolkit.cli.ih_elastic.cmd_snapshots.cmd_restore import cmd_restore
from infrahouse_toolkit.cli.ih_elastic.cmd_snapshots.cmd_status import cmd_status

LOG = logging.getLogger()


@click.group(name="snapshots")
def cmd_snapshots():
    """
    Work with snapshots.
    """


for cmd in [cmd_status, cmd_create_repository, cmd_delete_repository, cmd_create, cmd_restore]:
    # noinspection PyTypeChecker
    cmd_snapshots.add_command(cmd)



"""
.. topic:: ``ih-ec2 list``

    A ``ih-ec2 list`` subcommand.

    See ``ih-ec2 list`` for more details.
"""

import sys
from logging import getLogger
from pprint import pformat

import click
from botocore.exceptions import ClientError
from tabulate import tabulate

LOG = getLogger(__name__)


def list_ec2_instances(ec2_client):
    """
    Print a summary about EC2 instances in a region.
    """
    response = ec2_client.describe_instances()
    LOG.debug("describe_instances() = %s", pformat(response, indent=4))
    instances = []
    header = ["InstanceId", "InstanceType", "PublicDnsName", "PublicIpAddress", "PrivateIpAddress"]
    for reservation in response["Reservations"]:
        for instance in reservation["Instances"]:
            name = None
            for tag in instance.get("Tags") or []:
                if tag["Key"] == "Name":
                    name = tag["Value"]
            row = [name]
            for field in header:
                value = instance[field] if field in instance else ""
                row.append(value)
            row.append(instance["State"]["Name"])
            instances.append(row)

    print(tabulate(instances, headers=["Name"] + header + ["State"], tablefmt="outline"))


@click.command(name="list")
@click.pass_context
def cmd_list(ctx):
    """
    List created EC2 instances.
    """
    ec2_client = ctx.obj["ec2_client"]
    aws_config = ctx.obj["aws_config"]
    try:
        list_ec2_instances(ec2_client)
    except ClientError as err:
        LOG.exception(err)
        LOG.info("Try to run ih-ec2 with --aws-profile option.")
        LOG.info("Available profiles:\n\t%s", "\n\t".join(aws_config.profiles))
        sys.exit(1)



"""
.. topic:: ``ih-github runner is-registered``

    A ``ih-github runner is-registered`` subcommand.

    See ``ih-github runner is-registered --help`` for more details.
"""

import json
import logging
import sys

import click
from requests import get

LOG = logging.getLogger()


@click.command(
    name="is-registered",
)
@click.argument("name")
@click.pass_context
def cmd_is_registered(ctx, **kwargs):
    """
    Check if a runner with the given name is already registered.

    Exit code is zero if registered. Otherwise, 1.
    """
    github_token = ctx.obj["github_token"]
    org = ctx.obj["org"]
    response = get(
        f"https://api.github.com/orgs/{org}/actions/runners",
        headers={
            "Accept": "application/vnd.github+json",
            "Authorization": f"Bearer {github_token}",
            "X-GitHub-Api-Version": "2022-11-28",
        },
        timeout=30,
    )
    for runner in response.json()["runners"]:
        if runner["name"] == kwargs["name"]:
            LOG.info("Runner %s is registered", kwargs["name"])
            print(json.dumps(runner, indent=4))
            sys.exit(0)

    LOG.info("Runner %s is unregistered", kwargs["name"])
    sys.exit(1)



"""
.. topic:: ``ih-elastic snapshots create``

    A ``ih-elastic snapshots create`` subcommand.

    See ``ih-elastic snapshots create --help`` for more details.
"""

import json
import logging
from datetime import datetime, timezone

import click
from elasticsearch import NotFoundError
from elasticsearch.client import ClusterClient, SnapshotClient

from infrahouse_toolkit import DEFAULT_OPEN_ENCODING

LOG = logging.getLogger()


@click.command(name="create")
@click.option(
    "--bucket-name",
    help="If the repository doesn't exit, the tool will create one in an S# bucket with this name. "
    "By default, it will parse /etc/puppetlabs/facter/facts.d/custom.json "
    "and look for key .elasticsearch.snapshots_bucket",
    default=None,
)
@click.argument("repository-name")
@click.pass_context
def cmd_create(ctx, **kwargs):
    """
    Creates a snapshot in a repository.
    """
    client = ClusterClient(ctx.obj["es"])
    cluster_name = client.info(target="_all")["cluster_name"]
    client = SnapshotClient(ctx.obj["es"])
    now = datetime.now(timezone.utc).strftime("%Y-%m-%d_%H-%M-%S.%f")

    repo_name = kwargs["repository_name"]
    try:
        client.get_repository(name=repo_name)
    except NotFoundError:
        bucket = kwargs["bucket_name"]
        if bucket is None:
            with open("/etc/puppetlabs/facter/facts.d/custom.json", encoding=DEFAULT_OPEN_ENCODING) as f_desc:
                bucket = json.load(f_desc)["elasticsearch"]["snapshots_bucket"]

        client.create_repository(
            name=kwargs["repository_name"],
            settings={
                "client": "default",
                "bucket": bucket,
            },
            type="s3",
        )

    client.create(repository=repo_name, snapshot=f"{cluster_name}-{now}")



"""
.. topic:: ``ih-github runner list``

    A ``ih-github runner list`` subcommand.

    See ``ih-github runner list --help`` for more details.
"""

import json
import logging

import click
from requests import get

LOG = logging.getLogger()


@click.command(
    name="list",
)
@click.pass_context
def cmd_list(ctx, *args, **kwargs):
    """
    List self-hosted runners

    """
    LOG.debug("args = %s", args)
    LOG.debug("kwargs = %s", kwargs)
    LOG.debug(ctx.args)
    github_token = ctx.obj["github_token"]
    org = ctx.obj["org"]
    response = get(
        f"https://api.github.com/orgs/{org}/actions/runners",
        headers={
            "Accept": "application/vnd.github+json",
            "Authorization": f"Bearer {github_token}",
            "X-GitHub-Api-Version": "2022-11-28",
        },
        timeout=30,
    )
    print(json.dumps(response.json()))



import pytest

from infrahouse_toolkit.terraform.backends import TFS3Backend
from infrahouse_toolkit.terraform.status import RunOutput, RunResult, TFStatus


def test_eq():
    assert TFStatus(
        TFS3Backend("foo_backet", "path/to/tf.state"), True, RunResult(1, 1, 1), RunOutput("no stdout", None)
    ) == TFStatus(TFS3Backend("foo_backet", "path/to/tf.state"), True, RunResult(1, 1, 1), RunOutput("no stdout", None))


def test_eq_no_affrected_res():
    assert TFStatus(
        TFS3Backend("foo_backet", "path/to/tf.state"),
        True,
        RunResult(1, 1, 1),
        RunOutput("no stdout", None),
        affected_resources=RunResult(None, None, None),
    ) == TFStatus(TFS3Backend("foo_backet", "path/to/tf.state"), True, RunResult(1, 1, 1), RunOutput("no stdout", None))


# We don't want to compare stdout/stderr when checking if statuses are the same
@pytest.mark.parametrize(
    "output1, output2",
    [
        (RunOutput("stdout1", None), RunOutput("stdout2", None)),
        (RunOutput(None, "stderr1"), RunOutput(None, "stderr2")),
    ],
)
def test_eq_no_std(output1, output2):
    assert TFStatus(
        TFS3Backend("foo_backet", "path/to/tf.state"),
        True,
        RunResult(1, 1, 1),
        output1,
    ) == TFStatus(TFS3Backend("foo_backet", "path/to/tf.state"), True, RunResult(1, 1, 1), output2)


def test_neq():
    assert TFStatus(
        TFS3Backend("foo_backet", "path/to/tf.state"), True, RunResult(1, 1, 1), RunOutput("no stdout", None)
    ) != TFStatus(TFS3Backend("foo_backet", "path/to/tf.state"), True, RunResult(1, 0, 1), RunOutput("no stdout", None))



"""
.. topic:: ``ih-secrets list``

    A ``ih-secrets list`` subcommand.

    See ``ih-secrets list`` for more details.
"""

import sys
from logging import getLogger
from operator import itemgetter
from pprint import pformat

import click
from botocore.exceptions import ClientError
from tabulate import tabulate

LOG = getLogger(__name__)


def list_secrets(secretsmanager_client, all_fields=False):
    """
    Print a summary about EC2 instances in a region.
    """
    secrets = []
    fields = ["Name", "Description"]
    if all_fields:
        fields.extend(["ARN", "CreatedDate", "LastAccessedDate", "LastChangedDate", "SecretVersionsToStages"])

    next_token = None
    while True:
        kwargs = {"NextToken": next_token} if next_token else {}
        response = secretsmanager_client.list_secrets(**kwargs)
        LOG.debug("list_secrets() = %s", pformat(response, indent=4))
        for secret in response["SecretList"]:
            row = []
            for field in fields:
                row.append(secret.get(field))
            secrets.append(row)
        next_token = response.get("NextToken")

        if next_token is None:
            break

    print(tabulate(sorted(secrets, key=itemgetter(0)), headers=fields, tablefmt="outline"))


@click.command(name="list")
@click.option("--all", help="Show all secret properties.", is_flag=True, default=False, show_default=True)
@click.pass_context
def cmd_list(ctx, **kwargs):
    """
    List created EC2 instances.
    """
    secretsmanager_client = ctx.obj["secretsmanager_client"]
    aws_config = ctx.obj["aws_config"]
    try:
        list_secrets(secretsmanager_client, kwargs["all"])
    except ClientError as err:
        LOG.exception(err)
        LOG.error("Try to run ih-secrets with --aws-profile option.")
        LOG.error("Available profiles:\n\t%s", "\n\t".join(aws_config.profiles))
        sys.exit(1)



"""
.. topic:: ``ih-github``

    A ``ih-github`` command, GitHub helpers

    See ``ih-github --help`` for more details.
"""

from logging import getLogger

import click

from infrahouse_toolkit.cli.ih_github.cmd_run import cmd_run
from infrahouse_toolkit.cli.ih_github.cmd_runner import cmd_runner
from infrahouse_toolkit.logging import setup_logging

LOG = getLogger()


@click.group(
    "ih-github",
)
@click.option(
    "--debug",
    help="Enable debug logging.",
    is_flag=True,
    default=False,
    show_default=True,
)
def ih_github(**kwargs):  # pylint: disable=unused-argument
    """
    Various GitHub helper commands. See ih-github --help for details.
    """
    setup_logging(debug=kwargs["debug"])


for cmd in [cmd_run, cmd_runner]:
    # noinspection PyTypeChecker
    ih_github.add_command(cmd)



"""
.. topic:: ``ih-elastic cluster-health``

    A ``ih-elastic cluster-health`` subcommand.

    See ``ih-elastic cluster-health --help`` for more details.
"""

import json
import logging

import click
from elasticsearch.client import ClusterClient

LOG = logging.getLogger()


@click.command(name="cluster-health")
@click.pass_context
def cmd_cluster_health(ctx):
    """
    Connect to Elasticsearch host and print the cluster health.
    """
    client = ClusterClient(ctx.obj["es"])
    health = client.health()
    LOG.info(json.dumps(health.body, indent=4))



import pytest

from infrahouse_toolkit.cli.ih_plan.cmd_min_permissions import ActionList


def test_add_sorted():
    """Ensure sorted order"""
    actions1 = ActionList()
    actions1.add("foo")
    actions1.add("bar")

    actions2 = ActionList()
    actions2.add("bar")
    actions2.add("foo")

    assert actions1.actions == actions2.actions


def test_add():
    """add() an action and maintain uniqueness."""
    actions = ActionList()
    actions.add("foo")
    assert actions.actions == ["foo"]

    actions.add("foo")
    assert actions.actions == ["foo"]

    actions.add("bar")
    assert "bar" in actions.actions
    assert "foo" in actions.actions


@pytest.mark.parametrize(
    "action, dependent_actions",
    [
        (
            "autoscaling:CreateAutoScalingGroup",
            [
                "autoscaling:CreateAutoScalingGroup",
                "iam:PassRole",
                "iam:CreateServiceLinkedRole",
                "ec2:CreateTags",
                "ec2:RunInstances",
            ],
        ),
        (
            "logs:CreateLogGroup",
            [
                "logs:CreateLogGroup",
                "logs:TagResource",
            ],
        ),
        (
            "lambda:CreateFunction",
            [
                "lambda:CreateFunction",
                "lambda:TagResource",
            ],
        ),
        (
            "s3:PutObject",
            [
                "kms:Decrypt",
                "kms:CreateGrant",
                "kms:DescribeKey",
                "kms:Encrypt",
                "s3:PutObject",
                "s3:PutObjectTagging",
                "s3:AbortMultipartUpload",
                "s3:GetObject",
                "s3:ListMultipartUploadParts",
            ],
        ),
        (
            "events:PutRule",
            [
                "events:PutRule",
                "events:TagResource",
            ],
        ),
        (
            "events:PutTargets",
            [
                "events:PutTargets",
                "events:TagResource",
            ],
        ),
        (
            "iam:CreateInstanceProfile",
            [
                "iam:CreateInstanceProfile",
                "iam:TagInstanceProfile",
            ],
        ),
        (
            "iam:CreateInstanceProfile",
            [
                "iam:CreateInstanceProfile",
                "iam:TagInstanceProfile",
            ],
        ),
        (
            "s3:CreateBucket",
            [
                "s3:CreateBucket",
                "s3:PutBucketTagging",
            ],
        ),
    ],
)
def test_add_with_dependency(action, dependent_actions):
    actions = ActionList()
    actions.add(action)

    for dependency in dependent_actions:
        assert dependency in actions.actions


def test_add_with_rewrite():
    actions = ActionList()
    actions.add("auto scaling:foo")

    assert actions.actions == ["autoscaling:foo"]



"""
.. topic:: ``ih-github runner download``

    A ``ih-github runner download`` subcommand.

    See ``ih-github runner download --help`` for more details.
"""

import json
import logging
import sys

import click
from requests import get

LOG = logging.getLogger()


@click.command(
    name="download",
)
@click.option(
    "--os",
    help="Operating system",
    type=click.Choice(["linux", "osx", "win"]),
    default="linux",
    show_default=True,
)
@click.option(
    "--arch",
    help="Architecture",
    type=click.Choice(["arm", "arm64", "x64"]),
    default="x64",
    show_default=True,
)
@click.argument(
    "dest_file",
    required=False,
)
def cmd_download(**kwargs):
    """
    Download an actions-runner release tar-ball.
    """
    response = get(
        "https://api.github.com/repos/actions/runner/releases/latest",
        headers={
            "Accept": "application/vnd.github+json",
            "X-GitHub-Api-Version": "2022-11-28",
        },
        timeout=30,
    )
    response.raise_for_status()
    for asset in response.json()["assets"]:
        if f"-{kwargs['os']}-" in asset["name"] and f"-{kwargs['arch']}-" in asset["name"]:
            dest_file = kwargs["dest_file"] or asset["name"]
            LOG.debug("Saving %s in %s", asset["name"], dest_file)
            LOG.debug("Found asset: %s", json.dumps(asset, indent=4))
            download_file(asset["browser_download_url"], dest_file)
            LOG.info("Saved %s", dest_file)
            sys.exit(0)

    LOG.error("Actions runner for %s-%s not found.", kwargs["os"], kwargs["arch"])
    sys.exit(1)


def download_file(url: str, dst: str):
    """
    Download a file from a URL to a local destination file.

    :param url: URL of the file to download
    :param dst: Local path to save the downloaded file
    """
    # Make a GET request to the server
    with get(url, stream=True, timeout=30) as response:
        # Check if the request was successful
        response.raise_for_status()

        # Open the destination file in binary write mode
        with open(dst, "wb") as file:
            # Iterate over the content in chunks
            for chunk in response.iter_content(chunk_size=8192):
                # Write each chunk to the file
                if chunk:  # filter out keep-alive new chunks
                    file.write(chunk)



"""
.. topic:: ``ih-elastic snapshots delete-repository``

    A ``ih-elastic snapshots delete-repository`` subcommand.

    See ``ih-elastic snapshots delete-repository --help`` for more details.
"""

import logging

import click
from elasticsearch.client import SnapshotClient

LOG = logging.getLogger()


@click.command(name="delete-repository")
@click.argument("repository-name")
@click.pass_context
def cmd_delete_repository(ctx, **kwargs):
    """
    Deletes a repository.
    """
    client = SnapshotClient(ctx.obj["es"])
    client.delete_repository(
        name=kwargs["repository_name"],
    )



"""
.. topic:: ``ih-skeema``

    A ``ih-skeema`` command, Skeema helpers

    See ``ih-skeema --help`` for more details.
"""

import json
import sys
from logging import getLogger
from subprocess import check_output

import boto3
import click

from infrahouse_toolkit import DEFAULT_ENCODING
from infrahouse_toolkit.cli.ih_secrets.cmd_get import get_secret
from infrahouse_toolkit.cli.ih_skeema.cmd_run import cmd_run
from infrahouse_toolkit.logging import setup_logging

LOG = getLogger()


@click.group(
    "ih-skeema",
)
@click.option(
    "--debug",
    help="Enable debug logging.",
    is_flag=True,
    default=False,
    show_default=True,
)
@click.option(
    "--skeema-path",
    help="Path to the skeema executable.",
    default="skeema",
    show_default=True,
)
@click.option(
    "--username",
    help="Username to connect to database host",
    default="root",
    show_default=True,
)
@click.option(
    "--password",
    help="Password for database user. By default, read from environment variable $MYSQL_PWD.",
    envvar="MYSQL_PWD",
    show_default=True,
)
@click.option(
    "--credentials-secret",
    help="If specified, read username and password from AWS secrets manager. "
    "The secret value must be a JSON with keys 'username' and 'password'.",
    default=None,
)
@click.pass_context
def ih_skeema(ctx, **kwargs):  # pylint: disable=unused-argument
    """
    Various Skeema (https://www.skeema.io/) helper commands. See ih-skeema --help for details.
    """
    setup_logging(debug=kwargs["debug"])
    try:
        out = check_output(
            [kwargs["skeema_path"], "--version"],
        )
        LOG.debug("Running skeema command: %s", out.decode(DEFAULT_ENCODING))
        username = kwargs["username"]
        password = kwargs["password"]

        if kwargs["credentials_secret"]:
            secretsmanager_client = boto3.client("secretsmanager")
            credentials = json.loads(get_secret(secretsmanager_client, kwargs["credentials_secret"]))
            username = credentials["username"]
            password = credentials["password"]

        ctx.obj = {
            "skeema_path": kwargs["skeema_path"],
            "username": username,
            "password": password,
        }

    except FileNotFoundError as err:
        LOG.error("Skeema executable not found: %s", err)
        LOG.error("Please install Skeema first or specify a skeema executable via --skeema-path.")
        sys.exit(1)


for cmd in [cmd_run]:
    # noinspection PyTypeChecker
    ih_skeema.add_command(cmd)



"""Fixtures."""

from textwrap import dedent

import pytest


@pytest.fixture
def terraform_tf(tmp_path):
    """
    Return a temporary path with a Terraform backend configuration.

    :param tmp_path: pytest's tmp_path fixture.
    :type tmp_path: pathlib.Path
    :return: A temporary file with the Terraform backend configuration.
    """
    tf_backend = tmp_path / "terraform.tf"
    tf_backend.write_text(
        dedent(
            """
            terraform {
              backend "s3" {
                bucket = "infrahouse-foo"
                key    = "github.state"
              }
              required_providers {}
            }
            """
        )
    )
    return tf_backend



"""
InfraHouse Lock interface.
"""


class BaseLock:
    """
    Base Lock interface.

    Provides public properties and methods.
    """

    def __init__(self):
        self._name = None

    @property
    def name(self):
        """Descriptive for humans lock name."""
        return self._name

    @property
    def exclusive(self):
        """Whether the lock is exclusive as opposite of shared."""
        return True

    @property
    def shared(self):
        """Whether the lock is shared as opposite of exclusive."""
        return not self.exclusive



"""
.. topic:: ``ih-aws credentials``

    A ``ih-aws credentials`` subcommand.

    See ``ih-aws credentials --help`` for more details.
"""

import sys
from logging import getLogger

import click
from botocore.exceptions import ClientError

from infrahouse_toolkit.aws import get_aws_client

LOG = getLogger(__name__)


@click.command(name="credentials")
@click.option("--docker", help="Generate command line arguments for docker", is_flag=True, default=False)
@click.option("--export", "-e", help="Add export keyword", is_flag=True, default=False)
@click.pass_context
def cmd_credentials(ctx, docker, export):
    """
    Print AWS temporary credentials
    """
    aws_config = ctx.obj["aws_config"]
    aws_profile = ctx.obj["aws_profile"]
    try:
        aws_session = ctx.obj["aws_session"]
        creds = {
            "AWS_ACCESS_KEY_ID": aws_session.get_credentials().access_key,
            "AWS_SECRET_ACCESS_KEY": aws_session.get_credentials().secret_key,
        }

        if aws_config.get_region(aws_profile):
            creds["AWS_DEFAULT_REGION"] = aws_config.get_region(aws_profile)

        if aws_session.get_credentials().token:
            creds["AWS_SESSION_TOKEN"] = aws_session.get_credentials().token

        response = get_aws_client(
            "sts", aws_profile, aws_config.get_region(aws_profile), session=aws_session
        ).get_caller_identity()
        LOG.debug("Connected to AWS as %s", response["Arn"])

        if docker:
            pairs = [f"-e {k}={v}" for k, v in creds.items()]
            sys.stdout.write(" ".join(pairs))
        else:
            pairs = [f"{'export ' if export else ''}{k}={v}" for k, v in creds.items()]
            print("\n".join(pairs))

    except ClientError as err:
        LOG.exception(err)
        LOG.error("Try to run ih-secrets with --aws-profile option.")
        LOG.error("Available profiles:\n\t%s", "\n\t".join(aws_config.profiles))
        sys.exit(1)



"""Top level exceptions.

The exception hierarchy repeats the structure of the infrahouse_toolkit package.
Each module in the package has its own exceptions.py module.
The module exceptions are inherited from the upper module exceptions.

"""

from infrahouse_toolkit.exceptions import IHException


class IHAWSException(IHException):
    """AWS related InfraHouse exception"""
